@@ -17,6 +17,7 @@ from clu.tools import ActorHandler
 
 from jaeger import FPS, __version__, config, log
 from jaeger.commands import CommandID
+from jaeger.ieb import IEB
 from jaeger.maskbits import LowTemperature
 
 
@@ -52,7 +53,7 @@ class JaegerActor(clu.LegacyActor):
         log.addHandler(self.actor_handler)
         self.actor_handler.setLevel(logging.INFO)
 
-        if fps.ieb and not fps.ieb.disabled:
+        if isinstance(fps.ieb, IEB) and not fps.ieb.disabled:
             self.timed_commands.add_command("ieb status", delay=ieb_status_delay)
             asyncio.create_task(self.handle_temperature())
 
@@ -107,8 +108,10 @@ class JaegerActor(clu.LegacyActor):
                     "w",
                     text="Disabling very low temperature mode. Setting holding torque.",
                 )
-            await self.fps.send_to_all(
-                CommandID.SET_HOLDING_CURRENT, alpha=ht[0], beta=ht[1]
+            await self.fps.send_command(
+                CommandID.SET_HOLDING_CURRENT,
+                alpha=ht[0],
+                beta=ht[1],
             )
 
         sensor = config["low_temperature"]["sensor"]
@@ -118,6 +121,7 @@ class JaegerActor(clu.LegacyActor):
 
         while True:
             try:
+                assert isinstance(self.fps.ieb, IEB) and self.fps.ieb.disabled is False
                 device = self.fps.ieb.get_device(sensor)
                 temp = (await device.read())[0]
 
