@@ -19,10 +19,11 @@ import numpy
 import drift
 from sdsstools import read_yaml_file
 
-from jaeger import config, log, maskbits
+from jaeger import config, log
 from jaeger.commands import Command, CommandID
 from jaeger.exceptions import FPSLockedError, TrajectoryError
 from jaeger.ieb import IEB
+from jaeger.maskbits import PositionerStatus, ResponseCode
 from jaeger.utils import int_to_bytes
 
 
@@ -53,6 +54,8 @@ async def send_trajectory(
     fps: FPS,
     trajectories: str | pathlib.Path | TrajectoryDataType,
     use_sync_line=True,
+    send_trajectory=True,
+    start_trajectory=True,
 ) -> Trajectory:
     """Sends a set of trajectories to the positioners.
 
@@ -76,6 +79,13 @@ async def send_trajectory(
         If `True`, the SYNC line will be used to synchronise the beginning of
         all trajectories. Otherwise a `.START_TRAJECTORY` command will be sent
         over the CAN network.
+    send_trajectory
+        If `True`, sends the trajectory to the positioners and returns the
+        `.Trajectory` instance.
+    start_trajectory
+        If `True`, runs the trajectory after sending it. Otherwise, returns
+        the `.Trajectory` instance after sending the data. Ignored if
+        `send_trajectory=False`.
 
     Raises
     ------
@@ -101,23 +111,44 @@ async def send_trajectory(
 
     if use_sync_line:
         if not isinstance(fps.ieb, IEB) or fps.ieb.disabled:
-            raise TrajectoryError("IEB is not connected. Cannot use SYNC line.")
+            raise TrajectoryError("IEB is not connected. Cannot use SYNC line.", traj)
         if (await fps.ieb.get_device("sync").read())[0] == "closed":
-            raise TrajectoryError("The SYNC line is on high.")
+            raise TrajectoryError("The SYNC line is on high.", traj)
+
+    if send_trajectory is False:
+        return traj
 
     log.debug("sending trajectory data.")
-    await traj.send()
 
-    if traj.failed:
-        raise TrajectoryError("Something went wrong sending the trajectory.")
+    try:
+        await traj.send()
+    except TrajectoryError as err:
+        raise TrajectoryError(
+            f"Something went wrong sending the trajectory: {err}",
+            err.trajectory,
+        )
 
     log.debug(f"Trajectory successfully sent in {traj.data_send_time:1f} seconds.")
     log.info(f"Expected time to complete trajectory: {traj.move_time:.2f} seconds.")
 
+    if start_trajectory is False:
+        return traj
+
     log.info("starting trajectory ...")
-    result = await traj.start(use_sync_line=use_sync_line)
-    if traj.failed or not result:
-        raise TrajectoryError("Something went wrong starting the trajectory.")
+    try:
+        await traj.start(use_sync_line=use_sync_line)
+    except TrajectoryError as err:
+
+        if traj.start_time is not None and traj.end_time is not None:
+            elapsed = traj.end_time - traj.start_time
+            elapsed_msg = f" Trajectory failed {elapsed:.2f} seconds after start."
+        else:
+            elapsed_msg = ""
+
+        raise TrajectoryError(
+            f"Something went wrong during the trajectory: {err}.{elapsed_msg}",
+            err.trajectory,
+        )
 
     log.info("All positioners have successfully reached their positions.")
 
@@ -179,14 +210,20 @@ class Trajectory(object):
             )
 
         if self.fps.moving:
-            raise TrajectoryError("The FPS is moving. Cannot send new trajectory.")
+            raise TrajectoryError(
+                "The FPS is moving. Cannot send new trajectory.",
+                self,
+            )
 
         if isinstance(trajectories, (str, pathlib.Path)):
             self.trajectories = cast(TrajectoryDataType, read_yaml_file(trajectories))
         elif isinstance(trajectories, dict):
             self.trajectories = trajectories
         else:
-            raise TrajectoryError("invalid trajectory data.")
+            raise TrajectoryError("invalid trajectory data.", self)
+
+        # List of positioners that failed receiving the trajectory and reason.
+        self.failed_positioners: dict[int, str] = {}
 
         self.validate()
 
@@ -194,12 +231,20 @@ class Trajectory(object):
         self.n_points = {}
 
         #: The time required to complete the trajectory.
-        self.move_time = None
+        self.move_time: float | None = None
 
         #: How long it took to send the trajectory.
-        self.data_send_time = None
+        self.data_send_time: float | None = None
 
         self.failed = False
+        self.send_new_trajectory_failed = False
+
+        # Commands that will be sent. Mostly for inspection if the trajectory fails.
+        self.data_send_cmd: Command | None = None
+        self.end_traj_cmds: Command | None = None
+
+        self.start_time: float | None = None
+        self.end_time: float | None = None
 
         self._ready_to_start = False
 
@@ -207,22 +252,29 @@ class Trajectory(object):
         """Validates the trajectory."""
 
         if len(self.trajectories) == 0:
-            raise TrajectoryError("trajectory is empty.")
+            raise TrajectoryError("trajectory is empty.", self)
 
         if len(self.trajectories) != len(numpy.unique(list(self.trajectories))):
-            raise TrajectoryError("Duplicate positioner trajectories.")
+            raise TrajectoryError("Duplicate positioner trajectories.", self)
 
         for pid in self.trajectories:
             trajectory = self.trajectories[pid]
 
             if "alpha" not in trajectory or "beta" not in trajectory:
-                raise TrajectoryError(f"Positioner {pid} missing alpha or beta data.")
+                self.failed_positioners[pid] = "NO_DATA"
+                raise TrajectoryError(
+                    f"Positioner {pid} missing alpha or beta data.",
+                    self,
+                )
 
             for arm in ["alpha", "beta"]:
                 data = numpy.array(list(zip(*trajectory[arm]))[0])
 
                 # if numpy.any(data > 360) or numpy.any(data < 0):
-                #     raise TrajectoryError(f"Positioner {pid} has points out of range.")
+                #     raise TrajectoryError(
+                #         f"Positioner {pid}: trajectory has points out of range.",
+                #         self,
+                #     )
 
                 if arm == "beta":
                     if config.get("safe_mode", False):
@@ -231,9 +283,11 @@ class Trajectory(object):
                         else:
                             min_beta = config["safe_mode"]["min_beta"]
                         if numpy.any(data < min_beta):
+                            self.failed_positioners[pid] = "SAFE_MODE"
                             raise TrajectoryError(
                                 f"Positioner {pid}: safe mode is on "
-                                f"and beta < {min_beta}."
+                                f"and beta < {min_beta}.",
+                                self,
                             )
 
     async def send(self):
@@ -248,7 +302,7 @@ class Trajectory(object):
             timeout=1.0,
         ):
             self.failed = True
-            raise TrajectoryError("some positioners did not respond.")
+            raise TrajectoryError("some positioners did not respond.", self)
 
         # Check that all positioners are ready to receive a new trajectory.
         for pos_id in self.trajectories:
@@ -260,7 +314,8 @@ class Trajectory(object):
                 self.failed = True
                 raise TrajectoryError(
                     f"positioner_id={pos_id} is disabled but "
-                    "included in the trajectory."
+                    "included in the trajectory.",
+                    self,
                 )
 
             if (
@@ -269,8 +324,10 @@ class Trajectory(object):
                 or positioner.flags.DISPLACEMENT_COMPLETED not in status
             ):
                 self.failed = True
+                self.failed_positioners[pos_id] = "NOT_READY"
                 raise TrajectoryError(
-                    f"positioner_id={pos_id} is not ready to receive a trajectory."
+                    f"positioner_id={pos_id} is not ready to receive a trajectory.",
+                    self,
                 )
 
             traj_pos = self.trajectories[pos_id]
@@ -305,7 +362,8 @@ class Trajectory(object):
         )
 
         if new_traj_cmd.status.failed or new_traj_cmd.status.timed_out:
-            raise TrajectoryError("Failed sending SEND_NEW_TRAJECTORY.")
+            self.failed = True
+            raise TrajectoryError("Failed sending SEND_NEW_TRAJECTORY.", self)
 
         start_trajectory_send_time = time.time()
 
@@ -338,35 +396,41 @@ class Trajectory(object):
 
                     data[pos_id] = data_pos
 
-                data_send_cmd = await self.fps.send_command(
+                self.data_send_cmd = await self.fps.send_command(
                     "SEND_TRAJECTORY_DATA",
                     positioner_ids=send_trajectory_pids,
                     data=data,
                 )
 
-                if data_send_cmd.status.failed or data_send_cmd.status.timed_out:
+                status = self.data_send_cmd.status
+                if status.failed or status.timed_out:
+                    for reply in self.data_send_cmd.replies:
+                        if reply.response_code != ResponseCode.COMMAND_ACCEPTED:
+                            code = reply.response_code.name
+                            self.failed_positioners[reply.positioner_id] = code
                     self.failed = True
                     raise TrajectoryError(
-                        "At least one SEND_TRAJECTORY_COMMAND failed."
+                        "At least one SEND_TRAJECTORY_COMMAND failed.",
+                        self,
                     )
 
         # Finalise the trajectories
-        end_traj_cmds = await self.fps.send_command(
+        self.end_traj_cmds = await self.fps.send_command(
             "TRAJECTORY_DATA_END",
             positioner_ids=list(self.trajectories.keys()),
         )
 
-        for cmd in end_traj_cmds:
-
+        for cmd in self.end_traj_cmds:
             if cmd.status.failed:
                 self.failed = True
-                raise TrajectoryError("TRAJECTORY_DATA_END failed.")
+                raise TrajectoryError("TRAJECTORY_DATA_END failed.", self)
 
-            if maskbits.ResponseCode.INVALID_TRAJECTORY in cmd.replies[0].response_code:
+            if ResponseCode.INVALID_TRAJECTORY in cmd.replies[0].response_code:
                 self.failed = True
                 raise TrajectoryError(
                     f"positioner_id={cmd.positioner_id} got an "
-                    f"INVALID_TRAJECTORY reply."
+                    f"INVALID_TRAJECTORY reply.",
+                    self,
                 )
 
         self.data_send_time = time.time() - start_trajectory_send_time
@@ -380,7 +444,7 @@ class Trajectory(object):
         """Starts the trajectory."""
 
         if not self._ready_to_start or self.failed:
-            raise TrajectoryError("The trajectory has not been sent.")
+            raise TrajectoryError("The trajectory has not been sent.", self)
 
         for positioner_id in list(self.trajectories.keys()):
             self.fps[positioner_id].move_time = self.move_time
@@ -388,9 +452,12 @@ class Trajectory(object):
         if use_sync_line:
 
             if not isinstance(self.fps.ieb, IEB) or self.fps.ieb.disabled:
-                raise TrajectoryError("IEB is not connected. Cannot use SYNC line.")
+                raise TrajectoryError(
+                    "IEB is not connected. Cannot use SYNC line.",
+                    self,
+                )
             if (await self.fps.ieb.get_device("sync").read())[0] == "closed":
-                raise TrajectoryError("The SYNC line is on high.")
+                raise TrajectoryError("The SYNC line is on high.", self)
 
             sync = self.fps.ieb.get_device("sync")
             assert isinstance(sync, drift.Relay)
@@ -417,45 +484,51 @@ class Trajectory(object):
             if command.status.failed:
                 await self.fps.stop_trajectory()
                 self.failed = True
-                raise TrajectoryError("START_TRAJECTORY failed")
+                raise TrajectoryError("START_TRAJECTORY failed", self)
 
         restart_pollers = True if self.fps.pollers.running else False
         await self.fps.pollers.stop()
 
-        start_time = time.time()
+        self.start_time = time.time()
 
         try:
             if self.move_time is None:
-                raise TrajectoryError("move_time not set.")
+                raise TrajectoryError("move_time not set.", self)
 
             while True:
 
                 await asyncio.sleep(1)
 
                 if self.fps.locked:
-                    raise TrajectoryError("The FPS got locked during the trajectory.")
+                    raise TrajectoryError(
+                        "The FPS got locked during the trajectory.",
+                        self,
+                    )
 
                 await self.fps.update_status()
 
                 if all(
                     [
-                        maskbits.PositionerStatus.DISPLACEMENT_COMPLETED
-                        in self.fps[pid].status
+                        PositionerStatus.DISPLACEMENT_COMPLETED in self.fps[pid].status
                         for pid in self.trajectories
                     ]
                 ):
                     self.failed = False
                     break
 
-            elapsed = time.time() - start_time
+            elapsed = time.time() - self.start_time
             if elapsed > (self.move_time + 3):
-                raise TrajectoryError("Some positioners did not complete the move.")
+                raise TrajectoryError(
+                    "Some positioners did not complete the move.",
+                    self,
+                )
 
         except BaseException:
             self.failed = True
             raise
 
         finally:
+            self.end_time = time.time()
             if restart_pollers:
                 self.fps.pollers.start()
 
@@ -470,7 +543,7 @@ class Trajectory(object):
         )
 
         if cmd.status.failed:
-            raise TrajectoryError("Cannot abort trajectory transmission.")
+            raise TrajectoryError("Cannot abort trajectory transmission.", self)
 
 
 class SendNewTrajectory(Command):
