@@ -19,7 +19,7 @@ from typing import Any, ClassVar, Dict, List, Optional, Tuple, Type, TypeVar, Un
 import numpy
 
 from jaeger import can_log, config, log, start_file_loggers
-from jaeger.can import CANnetInterface, JaegerCAN
+from jaeger.can import JaegerCAN
 from jaeger.commands import (
     Command,
     CommandID,
@@ -33,7 +33,6 @@ from jaeger.exceptions import (
     JaegerError,
     JaegerUserWarning,
     PositionerError,
-    TrajectoryError,
 )
 from jaeger.ieb import IEB
 from jaeger.interfaces import BusABC
@@ -180,7 +179,7 @@ class FPS(BaseFPS):
 
         if isinstance(self.ieb, (str, dict)):
             if isinstance(self.ieb, str):
-                self.ieb = os.path.expanduser(os.path.expandvars(self.ieb))
+                self.ieb = os.path.expanduser(os.path.expandvars(str(self.ieb)))
                 if not os.path.isabs(self.ieb):
                     self.ieb = os.path.join(os.path.dirname(__file__), self.ieb)
             try:
@@ -320,31 +319,29 @@ class FPS(BaseFPS):
         # Get the positioner-to-bus map
         await self._get_positioner_bus_map()
 
-        # Resets all positioners
-        for positioner in self.positioners.values():
-            positioner.reset()
-
         # Stop poller in case they are running
         await self.pollers.stop()
 
-        get_firmware_command = self.send_command(
+        get_fw_command = self.send_command(
             CommandID.GET_FIRMWARE_VERSION,
             positioner_ids=0,
             timeout=config["fps"]["initialise_timeouts"],
         )
 
-        assert isinstance(get_firmware_command, GetFirmwareVersion)
-        await get_firmware_command
+        assert isinstance(get_fw_command, GetFirmwareVersion)
+        await get_fw_command
 
-        if get_firmware_command.status.failed:
+        if get_fw_command.status.failed:
             raise JaegerError("Failed retrieving firmware version.")
 
         # Loops over each reply and set the positioner status to OK. If the
         # positioner was not in the list, adds it.
-        for reply in get_firmware_command.replies:
+        ignored_positioners = []
+        for reply in get_fw_command.replies:
             if reply.positioner_id not in self.positioners:
 
                 if reply.positioner_id in config["fps"]["skip_positioners"]:
+                    ignored_positioners.append(reply.positioner_id)
                     continue
 
                 if hasattr(reply.message, "interface"):
@@ -357,7 +354,14 @@ class FPS(BaseFPS):
 
             positioner = self.positioners[reply.positioner_id]
             positioner.fps = self
-            positioner.firmware = get_firmware_command.get_firmware(reply.positioner_id)
+            positioner.firmware = get_fw_command.get_firmware()[reply.positioner_id]
+
+        if len(ignored_positioners) > 0:
+            warnings.warn(
+                "The following connected positioners are ignored as they are "
+                f"in the skip_positioners list: {ignored_positioners}",
+                JaegerUserWarning,
+            )
 
         pids = sorted(list(self.keys()))
         if len(pids) > 0:
@@ -414,13 +418,6 @@ class FPS(BaseFPS):
             )
             return self
 
-        if self.locked:
-            warnings.warn("FPS is locked. Trying to unlock it.", JaegerUserWarning)
-            if not await self.unlock():
-                raise JaegerError("FPS cannot be unlocked. Initialisation failed.")
-            else:
-                log.info("FPS unlocked successfully.")
-
         if config.get("safe_mode", False) is not False:
             min_beta = MIN_BETA
             if isinstance(config["safe_mode"], dict):
@@ -432,14 +429,19 @@ class FPS(BaseFPS):
 
         # Disable collision detection for listed robots.
         disable_collision = config["fps"]["disable_collision_detection_positioners"]
-        if len(disable_collision):
+        disable_connected = list(set(disable_collision) & set(self.positioners.keys()))
+        if len(disable_connected) > 0:
+            warnings.warn(
+                f"Disabling collision detection for positioners {disable_connected}.",
+                JaegerUserWarning,
+            )
             await self.send_command(
                 CommandID.ALPHA_CLOSED_LOOP_WITHOUT_COLLISION_DETECTION,
-                positioner_ids=disable_collision,
+                positioner_ids=disable_connected,
             )
             await self.send_command(
                 CommandID.BETA_CLOSED_LOOP_WITHOUT_COLLISION_DETECTION,
-                positioner_ids=disable_collision,
+                positioner_ids=disable_connected,
             )
 
         # Start the pollers
@@ -626,13 +628,13 @@ class FPS(BaseFPS):
 
         warnings.warn("Locking the FPS.", JaegerUserWarning)
 
-        self._locked = True
+        if stop_trajectories:
+            await self.stop_trajectory()
 
         if by:
             self.locked_by += by
 
-        if stop_trajectories:
-            await self.stop_trajectory()
+        self._locked = True
 
     async def unlock(self, force=False):
         """Unlocks the `.FPS` if all collisions have been resolved."""
@@ -778,27 +780,27 @@ class FPS(BaseFPS):
         else:
             n_positioners = None
 
-        get_firmware_command = self.send_command(
+        get_fw_command = self.send_command(
             CommandID.GET_FIRMWARE_VERSION,
             positioner_ids=0,
             timeout=timeout,
             n_positioners=n_positioners,
         )
 
-        assert isinstance(get_firmware_command, GetFirmwareVersion)
-        await get_firmware_command
+        assert isinstance(get_fw_command, GetFirmwareVersion)
+        await get_fw_command
 
-        if get_firmware_command.status.failed:
+        if get_fw_command.status.failed:
             log.error("Failed retrieving firmware version.")
             return False
 
-        for reply in get_firmware_command.replies:
+        for reply in get_fw_command.replies:
             pid = reply.positioner_id
             if pid not in self.positioners:
                 continue
 
             positioner = self.positioners[pid]
-            positioner.firmware = get_firmware_command.get_firmware(pid)
+            positioner.firmware = get_fw_command.get_firmware()[pid]
 
         return True
 
@@ -901,15 +903,17 @@ class FPS(BaseFPS):
         Returns
         -------
         trajectory
-            The `.Trajectory` object, or `None` if the trajectory failed.
+            The `.Trajectory` object.
+
+        Raises
+        ------
+        TajectoryError
+            You can inspect the `.Trajectory` object by capturing the error and
+            accessing ``error.trajectory``.
 
         """
 
-        try:
-            return await send_trajectory(self, *args, **kwargs)
-        except TrajectoryError as ee:
-            log.error(f"Sending trajectory failed with error: {ee}")
-            return None
+        return await send_trajectory(self, *args, **kwargs)
 
     def abort(self):
         """Aborts trajectories and stops positioners."""
@@ -930,8 +934,6 @@ class FPS(BaseFPS):
     async def report_status(self) -> Dict[str, Any]:
         """Returns a dict with the position and status of each positioner."""
 
-        assert isinstance(self.can, CANnetInterface)
-
         status: Dict[str, Any] = {"positioners": {}}
 
         for positioner in self.positioners.values():
@@ -948,7 +950,7 @@ class FPS(BaseFPS):
             }
 
         try:
-            status["devices"] = self.can.device_status
+            status["devices"] = self.can.device_status  # type: ignore
         except AttributeError:
             pass
 
@@ -965,11 +967,7 @@ class FPS(BaseFPS):
     async def shutdown(self):
         """Stops pollers and shuts down all remaining tasks."""
 
-        bootloader = all(
-            [positioner.is_bootloader() is True for positioner in self.values()]
-        )
-
-        if not bootloader:
+        if not self.is_bootloader:
             log.info("Stopping positioners and shutting down.")
             await self.stop_trajectory()
 
@@ -977,8 +975,6 @@ class FPS(BaseFPS):
         if self.pollers:
             await self.pollers.stop()
 
-        await asyncio.sleep(1)
-
         log.debug("Cancelling all pending tasks and shutting down.")
 
         tasks = [
