@@ -308,7 +308,7 @@ class Positioner(StatusMixIn):
             return True
 
         if not self.initialised:
-            raise PositionerError("failed inisialising.")
+            raise PositionerError("failed initialising.")
 
         # Update position only if it's not bootloader.
         await self.update_position()
@@ -338,7 +338,7 @@ class Positioner(StatusMixIn):
 
         assert isinstance(command, GetFirmwareVersion)
 
-        firmware = command.get_firmware(positioner_id=self.positioner_id)
+        firmware = command.get_firmware()[self.positioner_id]
         assert firmware is not None and isinstance(firmware, str)
 
         self.firmware = firmware
@@ -619,7 +619,7 @@ class Positioner(StatusMixIn):
                 (GotoAbsolutePosition, GotoRelativePosition),
             )
 
-            alpha_time, beta_time = goto_command.get_move_time()[0]
+            alpha_time, beta_time = goto_command.get_move_time()[self.positioner_id]
             move_time = max([alpha_time, beta_time])
             self._log(f"The move will take {move_time:.2f} seconds", logging.INFO)
 
@@ -731,3 +731,29 @@ class Positioner(StatusMixIn):
             f"<Positioner (id={self.positioner_id}, "
             f"status={self.status!s}, initialised={self.initialised})>"
         )
+
+    async def get_number_trajectories(self) -> int | None:
+        """Returns the number of trajectories executed by the positioner.
+
+        Will return `None` if the firmware does not support the
+        ``GET_NUMBER_TRAJECTORIES``.
+
+        """
+
+        if not self.firmware:
+            return None
+
+        if StrictVersion(self.firmware) < StrictVersion("04.01.21"):
+            return None
+
+        assert self.fps
+
+        cmd = await self.fps.send_command(
+            CommandID.GET_NUMBER_TRAJECTORIES,
+            positioner_ids=[self.positioner_id],
+            timeout=1,
+        )
+
+        n_traj = cmd.get_replies()[self.positioner_id]
+
+        return n_traj
